@@ -13,18 +13,14 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __DH_APP_H__
-#define __DH_APP_H__
+#ifndef DH_APP_H
+#define DH_APP_H
 
 #include <gtk/gtk.h>
 
-#include "dh-book-manager.h"
-
 G_BEGIN_DECLS
 
 #define DH_TYPE_APP         (dh_app_get_type ())
@@ -49,20 +45,19 @@ GType dh_app_get_type (void) G_GNUC_CONST;
 
 DhApp         *dh_app_new               (void);
 
-DhBookManager *dh_app_peek_book_manager (DhApp *self);
-GtkWindow     *dh_app_peek_first_window (DhApp *self);
-GtkWindow     *dh_app_peek_assistant    (DhApp *self);
+GtkWindow     *dh_app_peek_first_window (DhApp *app);
+GtkWindow     *dh_app_peek_assistant    (DhApp *app);
 
-void           dh_app_new_window        (DhApp *self);
-void           dh_app_quit              (DhApp *self);
-void           dh_app_search            (DhApp *self,
+void           dh_app_new_window        (DhApp *app);
+void           dh_app_quit              (DhApp *app);
+void           dh_app_search            (DhApp *app,
                                          const gchar *keyword);
-void           dh_app_search_assistant  (DhApp *self,
+void           dh_app_search_assistant  (DhApp *app,
                                          const gchar *keyword);
-void           dh_app_raise             (DhApp *self);
+void           dh_app_raise             (DhApp *app);
 
 gboolean      _dh_app_has_app_menu      (DhApp *app);
 
 G_END_DECLS
 
-#endif /* __DH_APP_H__ */
+#endif /* DH_APP_H */
