@@ -393,7 +393,7 @@ dh_assistant_view_set_link (DhAssistantView *view,
                         stylesheet_html,
                         javascript_html,
                         function,
-                        dh_link_get_type_as_string (link),
+                        dh_link_type_to_string (dh_link_get_link_type (link)),
                         dh_link_get_uri (link),
                         dh_link_get_name (link),
                         _("Book:"),
