@@ -28,6 +28,17 @@
 #include "dh-parser.h"
 #include "dh-util.h"
 
+/**
+ * SECTION:dh-book
+ * @Title: DhBook
+ * @Short_description: A book, usually the documentation for one library
+ *
+ * A #DhBook usually contains the documentation for one library (or
+ * application), for example GLib or GTK+. There is one #DhBook for each index
+ * file found and parsed (an index file is a file with the extension `*.devhelp`
+ * or `*.devhelp2`).
+ */
+
 /* Timeout to wait for new events in the book so that
  * they are merged and we don't spam unneeded signals */
 #define EVENT_MERGE_TIMEOUT_SECS 2
@@ -38,7 +49,7 @@ enum {
         BOOK_DISABLED,
         BOOK_UPDATED,
         BOOK_DELETED,
-        BOOK_LAST_SIGNAL
+        N_SIGNALS
 };
 
 typedef enum {
@@ -49,7 +60,7 @@ typedef enum {
 
 /* Structure defining basic contents to store about every book */
 typedef struct {
-        gchar        *path;
+        gchar        *index_file_path;
         gchar        *name;
         gchar        *title;
         gchar        *language;
@@ -77,15 +88,7 @@ typedef struct {
 
 G_DEFINE_TYPE_WITH_PRIVATE (DhBook, dh_book, G_TYPE_OBJECT);
 
-static void    book_monitor_event_cb (GFileMonitor      *file_monitor,
-                                      GFile             *file,
-                                      GFile             *other_file,
-                                      GFileMonitorEvent  event_type,
-                                      gpointer           user_data);
-static void    unref_node_link       (GNode             *node,
-                                      gpointer           data);
-
-static guint signals[BOOK_LAST_SIGNAL] = { 0 };
+static guint signals[N_SIGNALS] = { 0 };
 
 static void
 dh_book_dispose (GObject *object)
@@ -111,27 +114,13 @@ dh_book_finalize (GObject *object)
 
         priv = dh_book_get_instance_private (DH_BOOK (object));
 
-        if (priv->tree) {
-                g_node_traverse (priv->tree,
-                                 G_IN_ORDER,
-                                 G_TRAVERSE_ALL,
-                                 -1,
-                                 (GNodeTraverseFunc)unref_node_link,
-                                 NULL);
-                g_node_destroy (priv->tree);
-        }
-
+        _dh_util_free_book_tree (priv->tree);
         g_list_free_full (priv->keywords, (GDestroyNotify)dh_link_unref);
-
         g_list_free_full (priv->completions, g_free);
-
         g_free (priv->language);
-
         g_free (priv->title);
-
         g_free (priv->name);
-
-        g_free (priv->path);
+        g_free (priv->index_file_path);
 
         G_OBJECT_CLASS (dh_book_parent_class)->finalize (object);
 }
@@ -146,57 +135,53 @@ dh_book_class_init (DhBookClass *klass)
 
         /**
          * DhBook::enabled:
-         * @book: the book on which the signal is emitted
+         * @book: the book on which the signal is emitted.
          */
         signals[BOOK_ENABLED] =
                 g_signal_new ("enabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               0,
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
                               0);
 
         /**
          * DhBook::disabled:
-         * @book: the book on which the signal is emitted
+         * @book: the book on which the signal is emitted.
          */
         signals[BOOK_DISABLED] =
                 g_signal_new ("disabled",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               0,
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
                               0);
 
         /**
          * DhBook::updated:
-         * @book: the book on which the signal is emitted
+         * @book: the book on which the signal is emitted.
          */
         signals[BOOK_UPDATED] =
                 g_signal_new ("updated",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               0,
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
                               0);
 
         /**
          * DhBook::deleted:
-         * @book: the book on which the signal is emitted
+         * @book: the book on which the signal is emitted.
          */
         signals[BOOK_DELETED] =
                 g_signal_new ("deleted",
                               G_TYPE_FROM_CLASS (klass),
                               G_SIGNAL_RUN_LAST,
                               0,
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__VOID,
+                              NULL, NULL, NULL,
                               G_TYPE_NONE,
                               0);
 }
@@ -206,97 +191,8 @@ dh_book_init (DhBook *book)
 {
         DhBookPrivate *priv = dh_book_get_instance_private (book);
 
-        priv->name = NULL;
-        priv->path = NULL;
-        priv->title = NULL;
         priv->enabled = TRUE;
-        priv->tree = NULL;
-        priv->keywords = NULL;
-        priv->completions = NULL;
-        priv->monitor = NULL;
         priv->monitor_event = BOOK_MONITOR_EVENT_NONE;
-        priv->monitor_event_timeout_id = 0;
-}
-
-static void
-unref_node_link (GNode    *node,
-                 gpointer  data)
-{
-        dh_link_unref (node->data);
-}
-
-/**
- * dh_book_new:
- * @book_path: the path of the book
- *
- * Create a new #DhBook object.
- *
- * Returns: a new #DhBook object
- */
-DhBook *
-dh_book_new (const gchar *book_path)
-{
-        DhBookPrivate *priv;
-        DhBook     *book;
-        GError     *error = NULL;
-        GFile      *book_path_file;
-        gchar      *language;
-
-        g_return_val_if_fail (book_path, NULL);
-
-        book = g_object_new (DH_TYPE_BOOK, NULL);
-        priv = dh_book_get_instance_private (book);
-
-        /* Parse file storing contents in the book struct */
-        if (!dh_parser_read_file  (book_path,
-                                   &priv->title,
-                                   &priv->name,
-                                   &language,
-                                   &priv->tree,
-                                   &priv->keywords,
-                                   &error)) {
-                g_warning ("Failed to read '%s': %s",
-                           book_path, error->message);
-                g_error_free (error);
-
-                /* Deallocate the book, as we are not going to add it
-                 *  in the manager */
-                g_object_unref (book);
-                return NULL;
-        }
-
-        /* Store path */
-        priv->path = g_strdup (book_path);
-
-        /* Rewrite language, if any, including the prefix we want
-         * to use when seeing it. It is pretty ugly to do it here,
-         * but it's the only way of making sure we standarize how
-         * the language group is shown */
-        dh_util_ascii_strtitle (language);
-        priv->language = (language ?
-                          g_strdup_printf (_("Language: %s"), language) :
-                          g_strdup (_("Language: Undefined")));
-        g_free (language);
-
-        /* Setup monitor for changes */
-        book_path_file = g_file_new_for_path (book_path);
-        priv->monitor = g_file_monitor_file (book_path_file,
-                                             G_FILE_MONITOR_NONE,
-                                             NULL,
-                                             NULL);
-        if (priv->monitor) {
-                /* Setup changed signal callback */
-                g_signal_connect (priv->monitor,
-                                  "changed",
-                                  G_CALLBACK (book_monitor_event_cb),
-                                  book);
-        } else {
-                g_warning ("Couldn't setup monitoring of changes in book '%s'",
-                           priv->title);
-        }
-        g_object_unref (book_path_file);
-
-        return book;
 }
 
 static gboolean
@@ -373,13 +269,88 @@ book_monitor_event_cb (GFileMonitor      *file_monitor,
         }
 }
 
+/**
+ * dh_book_new:
+ * @index_file_path: the path to the index file.
+ *
+ * Returns: a new #DhBook object.
+ */
+DhBook *
+dh_book_new (const gchar *index_file_path)
+{
+        DhBookPrivate *priv;
+        DhBook     *book;
+        GFile      *index_file;
+        gchar      *language = NULL;
+        GError     *error = NULL;
+
+        g_return_val_if_fail (index_file_path, NULL);
+
+        book = g_object_new (DH_TYPE_BOOK, NULL);
+        priv = dh_book_get_instance_private (book);
+
+        index_file = g_file_new_for_path (index_file_path);
+
+        /* Parse file storing contents in the book struct */
+        if (!dh_parser_read_file (index_file,
+                                  &priv->title,
+                                  &priv->name,
+                                  &language,
+                                  &priv->tree,
+                                  &priv->keywords,
+                                  &error)) {
+                if (error != NULL) {
+                        g_warning ("Failed to read '%s': %s",
+                                   index_file_path,
+                                   error->message);
+                        g_error_free (error);
+                }
+
+                /* Deallocate the book, as we are not going to add it in the
+                 * manager.
+                 */
+                g_object_unref (book);
+                return NULL;
+        }
+
+        priv->index_file_path = g_strdup (index_file_path);
+
+        /* Rewrite language, if any, including the prefix we want
+         * to use when seeing it. It is pretty ugly to do it here,
+         * but it's the only way of making sure we standarize how
+         * the language group is shown */
+        dh_util_ascii_strtitle (language);
+        priv->language = (language != NULL ?
+                          g_strdup_printf (_("Language: %s"), language) :
+                          g_strdup (_("Language: Undefined")));
+        g_free (language);
+
+        /* Setup monitor for changes */
+        priv->monitor = g_file_monitor_file (index_file,
+                                             G_FILE_MONITOR_NONE,
+                                             NULL,
+                                             NULL);
+        if (priv->monitor != NULL) {
+                g_signal_connect (priv->monitor,
+                                  "changed",
+                                  G_CALLBACK (book_monitor_event_cb),
+                                  book);
+        } else {
+                g_warning ("Couldn't setup monitoring of changes in book '%s'",
+                           priv->title);
+        }
+
+        g_object_unref (index_file);
+        return book;
+}
+
 /**
  * dh_book_get_keywords:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the #GList of keywords associated with the book.
- *
- * Returns: (element-type utf8) (transfer none) (nullable): the keywords associated with the book
+ * Returns: (element-type DhLink) (transfer none) (nullable): the list of
+ * <emphasis>all</emphasis> #DhLink's part of @book, or %NULL if the book is
+ * disabled.
  */
 GList *
 dh_book_get_keywords (DhBook *book)
@@ -395,11 +366,10 @@ dh_book_get_keywords (DhBook *book)
 
 /**
  * dh_book_get_completions:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the #GList of completions associated with the book.
- *
- * Returns: (element-type utf8) (transfer full) (nullable): the completions associated with the book
+ * Returns: (element-type utf8) (transfer none) (nullable): the completions
+ * associated with the book.
  */
 GList *
 dh_book_get_completions (DhBook *book)
@@ -413,27 +383,25 @@ dh_book_get_completions (DhBook *book)
         if (!priv->enabled)
                 return NULL;
 
-        if (!priv->completions) {
+        if (priv->completions == NULL) {
                 GList *l;
-                for (l = priv->keywords; l; l = g_list_next (l)) {
+
+                for (l = priv->keywords; l != NULL; l = l->next) {
                         DhLink *link = l->data;
+                        gchar *str;
 
                         /* Add additional "page:" and "book:" completions */
                         if (dh_link_get_link_type (link) == DH_LINK_TYPE_BOOK) {
-                                priv->completions =
-                                        g_list_prepend (priv->completions,
-                                                        g_strdup_printf ("book:%s",
-                                                                         dh_link_get_name (link)));
+                                str = g_strdup_printf ("book:%s", dh_link_get_name (link));
+                                priv->completions = g_list_prepend (priv->completions, str);
                         }
                         else if (dh_link_get_link_type (link) == DH_LINK_TYPE_PAGE) {
-                                priv->completions =
-                                        g_list_prepend (priv->completions,
-                                                        g_strdup_printf ("page:%s",
-                                                                         dh_link_get_name (link)));
+                                str = g_strdup_printf ("page:%s", dh_link_get_name (link));
+                                priv->completions = g_list_prepend (priv->completions, str);
                         }
 
-                        priv->completions =  g_list_prepend (priv->completions,
-                                                          g_strdup (dh_link_get_name (link)));
+                        str = g_strdup (dh_link_get_name (link));
+                        priv->completions = g_list_prepend (priv->completions, str);
                 }
         }
 
@@ -442,11 +410,16 @@ dh_book_get_completions (DhBook *book)
 
 /**
  * dh_book_get_tree:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the #GNode representing the book tree.
+ * Gets the general structure of the book, as a tree. The tree contains only
+ * #DhLink's of type %DH_LINK_TYPE_BOOK or %DH_LINK_TYPE_PAGE. The other
+ * #DhLink's are not contained in the tree. To have a list of
+ * <emphasis>all</emphasis> #DhLink's part of the book, you need to call
+ * dh_book_get_keywords().
  *
- * Returns: (transfer none) (nullable): the book tree
+ * Returns: (transfer none) (nullable): the tree of #DhLink's part of the @book,
+ * or %NULL if the book is disabled.
  */
 GNode *
 dh_book_get_tree (DhBook *book)
@@ -462,11 +435,9 @@ dh_book_get_tree (DhBook *book)
 
 /**
  * dh_book_get_name:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the book name.
- *
- * Returns: the book name
+ * Returns: the book name.
  */
 const gchar *
 dh_book_get_name (DhBook *book)
@@ -482,11 +453,9 @@ dh_book_get_name (DhBook *book)
 
 /**
  * dh_book_get_title:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the book title.
- *
- * Returns: the book title
+ * Returns: the book title.
  */
 const gchar *
 dh_book_get_title (DhBook *book)
@@ -502,11 +471,9 @@ dh_book_get_title (DhBook *book)
 
 /**
  * dh_book_get_language:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the book language.
- *
- * Returns: the book language
+ * Returns: the book language.
  */
 const gchar *
 dh_book_get_language (DhBook *book)
@@ -522,11 +489,9 @@ dh_book_get_language (DhBook *book)
 
 /**
  * dh_book_get_path:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the book path.
- *
- * Returns: the book path
+ * Returns: the path to the index file.
  */
 const gchar *
 dh_book_get_path (DhBook *book)
@@ -537,16 +502,14 @@ dh_book_get_path (DhBook *book)
 
         priv = dh_book_get_instance_private (book);
 
-        return priv->path;
+        return priv->index_file_path;
 }
 
 /**
  * dh_book_get_enabled:
- * @book: a #DhBook object
+ * @book: a #DhBook.
  *
- * Get the current state of the book.
- *
- * Returns: %TRUE if the book is enabled
+ * Returns: whether the book is enabled.
  */
 gboolean
 dh_book_get_enabled (DhBook *book)
@@ -562,10 +525,10 @@ dh_book_get_enabled (DhBook *book)
 
 /**
  * dh_book_set_enabled:
- * @book: a #DhBook object
- * @enabled: %TRUE to enable the book
+ * @book: a #DhBook.
+ * @enabled: the new value.
  *
- * Enable or disable completely the book.
+ * Enables or disables the book.
  */
 void
 dh_book_set_enabled (DhBook   *book,
@@ -586,12 +549,13 @@ dh_book_set_enabled (DhBook   *book,
 
 /**
  * dh_book_cmp_by_path:
- * @a: a #DhBook object
- * @b: a second #DhBook object
+ * @a: a #DhBook.
+ * @b: a #DhBook.
  *
- * Compare the #DhBook @a and @b by their paths.
+ * Compares the #DhBook's by their path to the index file.
  *
- * Returns: an integer less than, equal to, or greater than zero, if @a is <, == or > than @b.
+ * Returns: an integer less than, equal to, or greater than zero, if @a is <, ==
+ * or > than @b.
  */
 gint
 dh_book_cmp_by_path (DhBook *a,
@@ -606,40 +570,18 @@ dh_book_cmp_by_path (DhBook *a,
         priv_a = dh_book_get_instance_private (a);
         priv_b = dh_book_get_instance_private (b);
 
-        return g_strcmp0 (priv_a->path, priv_b->path);
-}
-
-/**
- * dh_book_cmp_by_path_str:
- * @a: a #DhBook object
- * @b_path: a path to a book
- *
- * Compare the path of the #DhBook a with @b_path.
- *
- * Returns: an integer less than, equal to, or greater than zero, if the path of @a is <, == or > than @b_path.
- */
-gint
-dh_book_cmp_by_path_str (DhBook      *a,
-                         const gchar *b_path)
-{
-        DhBookPrivate *priv_a;
-
-        if (a == NULL)
-                return -1;
-
-        priv_a = dh_book_get_instance_private (a);
-
-        return g_strcmp0 (priv_a->path, b_path);
+        return g_strcmp0 (priv_a->index_file_path, priv_b->index_file_path);
 }
 
 /**
  * dh_book_cmp_by_name:
- * @a: a #DhBook object
- * @b: a second #DhBook object
+ * @a: a #DhBook.
+ * @b: a #DhBook.
  *
- * Compare the #DhBook @a and @b by their name.
+ * Compares the #DhBook's by their name.
  *
- * Returns: an integer less than, equal to, or greater than zero, if @a is <, == or > than @b.
+ * Returns: an integer less than, equal to, or greater than zero, if @a is <, ==
+ * or > than @b.
  */
 gint
 dh_book_cmp_by_name (DhBook *a,
@@ -660,40 +602,15 @@ dh_book_cmp_by_name (DhBook *a,
         return g_ascii_strcasecmp (priv_a->name, priv_b->name);
 }
 
-/**
- * dh_book_cmp_by_name_str:
- * @a: a #DhBook object
- * @b_name: a path to a book
- *
- * Compare the name of the #DhBook a with @b_name.
- *
- * Returns: an integer less than, equal to, or greater than zero, if the name of @a is <, == or > than @b_name.
- */
-gint
-dh_book_cmp_by_name_str (DhBook      *a,
-                         const gchar *b_name)
-{
-        DhBookPrivate *priv_a;
-
-        if (a == NULL)
-                return -1;
-
-        priv_a = dh_book_get_instance_private (a);
-
-        if (priv_a->name == NULL || b_name == NULL)
-                return -1;
-
-        return g_ascii_strcasecmp (priv_a->name, b_name);
-}
-
 /**
  * dh_book_cmp_by_title:
- * @a: a #DhBook object
- * @b: a second #DhBook object
+ * @a: a #DhBook.
+ * @b: a #DhBook.
  *
- * Compare the #DhBook @a and @b by their title.
+ * Compares the #DhBook's by their title.
  *
- * Returns: an integer less than, equal to, or greater than zero, if @a is <, == or > than @b.
+ * Returns: an integer less than, equal to, or greater than zero, if @a is <, ==
+ * or > than @b.
  */
 gint
 dh_book_cmp_by_title (DhBook *a,
