@@ -43,13 +43,12 @@ struct _DhLink {
         gchar       *book_id;
         gchar       *base_path;
 
+        DhLink      *book;
+
         gchar       *name;
         gchar       *name_collation_key;
         gchar       *relative_url;
 
-        DhLink      *book;
-        DhLink      *page;
-
         guint        ref_count;
 
         DhLinkType   type : 8;
@@ -62,18 +61,14 @@ G_DEFINE_BOXED_TYPE (DhLink, dh_link,
 static void
 link_free (DhLink *link)
 {
-        g_free (link->base_path);
         g_free (link->book_id);
+        g_free (link->base_path);
         g_free (link->name);
-        g_free (link->relative_url);
         g_free (link->name_collation_key);
+        g_free (link->relative_url);
 
-        if (link->book) {
+        if (link->book != NULL)
                 dh_link_unref (link->book);
-        }
-        if (link->page) {
-                dh_link_unref (link->page);
-        }
 
         g_slice_free (DhLink, link);
 }
@@ -85,8 +80,6 @@ link_free (DhLink *link)
  * @book_id: (nullable): the book ID, or %NULL.
  * @name: the name of the link.
  * @book: (nullable): the book that the link is contained in, or %NULL.
- * @page: (nullable): the page that the link is contained in, or %NULL. This
- * parameter is actually broken.
  * @relative_url: the URL relative to the book @base_path. Can contain an
  * anchor.
  *
@@ -95,8 +88,7 @@ link_free (DhLink *link)
  * @base_path and @book_id must be provided only for a link of type
  * %DH_LINK_TYPE_BOOK.
  *
- * If @type is not a #DH_LINK_TYPE_BOOK and not a #DH_LINK_TYPE_PAGE, then the
- * @book and @page links must be provided.
+ * If @type is not a #DH_LINK_TYPE_BOOK, then the @book link must be provided.
  *
  * @name and @relative_url must always be provided.
  *
@@ -106,26 +98,25 @@ DhLink *
 dh_link_new (DhLinkType   type,
              const gchar *base_path,
              const gchar *book_id,
-             const gchar *name,
              DhLink      *book,
-             DhLink      *page,
+             const gchar *name,
              const gchar *relative_url)
 {
         DhLink *link;
 
-        g_return_val_if_fail (name != NULL, NULL);
-        g_return_val_if_fail (relative_url != NULL, NULL);
-
         if (type == DH_LINK_TYPE_BOOK) {
                 g_return_val_if_fail (base_path != NULL, NULL);
                 g_return_val_if_fail (book_id != NULL, NULL);
-        }
-        if (type != DH_LINK_TYPE_BOOK &&
-            type != DH_LINK_TYPE_PAGE) {
+                g_return_val_if_fail (book == NULL, NULL);
+        } else {
+                g_return_val_if_fail (base_path == NULL, NULL);
+                g_return_val_if_fail (book_id == NULL, NULL);
                 g_return_val_if_fail (book != NULL, NULL);
-                g_return_val_if_fail (page != NULL, NULL);
         }
 
+        g_return_val_if_fail (name != NULL, NULL);
+        g_return_val_if_fail (relative_url != NULL, NULL);
+
         link = g_slice_new0 (DhLink);
 
         link->ref_count = 1;
@@ -136,75 +127,15 @@ dh_link_new (DhLinkType   type,
                 link->book_id = g_strdup (book_id);
         }
 
+        if (book != NULL)
+                link->book = dh_link_ref (book);
+
         link->name = g_strdup (name);
         link->relative_url = g_strdup (relative_url);
 
-        if (book != NULL) {
-                link->book = dh_link_ref (book);
-        }
-        if (page != NULL) {
-                link->page = dh_link_ref (page);
-        }
-
         return link;
 }
 
-/**
- * dh_link_compare:
- * @a: (type DhLink): a #DhLink.
- * @b: (type DhLink): a #DhLink.
- *
- * Compares the links @a and @b. This function is used to determine in which
- * order the links should be displayed.
- *
- * Returns: an integer less than zero if @a should appear before @b; zero if
- * there are no preferences; an integer greater than zero if @b should appear
- * before @a.
- */
-gint
-dh_link_compare (gconstpointer a,
-                 gconstpointer b)
-{
-        DhLink *la = (DhLink *) a;
-        DhLink *lb = (DhLink *) b;
-        gint    flags_diff;
-        gint    diff;
-
-        /* Sort deprecated hits last. */
-        flags_diff = (la->flags & DH_LINK_FLAGS_DEPRECATED) -
-                (lb->flags & DH_LINK_FLAGS_DEPRECATED);
-        if (flags_diff != 0) {
-                return flags_diff;
-        }
-
-        /* Collation-based sorting */
-        if (G_UNLIKELY (la->name_collation_key == NULL))
-                la->name_collation_key = g_utf8_collate_key (la->name, -1);
-        if (G_UNLIKELY (lb->name_collation_key == NULL))
-                lb->name_collation_key = g_utf8_collate_key (lb->name, -1);
-
-        diff = strcmp (la->name_collation_key,
-                       lb->name_collation_key);
-
-        /* For the same names, sort page links before other links. The page is
-         * more important than a symbol (typically contained in that page).
-         */
-        if (diff == 0) {
-                if (la->type == lb->type)
-                        return 0;
-
-                if (la->type == DH_LINK_TYPE_PAGE)
-                        return -1;
-
-                if (lb->type == DH_LINK_TYPE_PAGE)
-                        return 1;
-
-                return 0;
-        }
-
-        return diff;
-}
-
 /**
  * dh_link_ref:
  * @link: a #DhLink.
@@ -240,6 +171,50 @@ dh_link_unref (DhLink *link)
         }
 }
 
+/**
+ * dh_link_get_link_type:
+ * @link: a #DhLink.
+ *
+ * Returns: the #DhLinkType of @link.
+ */
+DhLinkType
+dh_link_get_link_type (DhLink *link)
+{
+        g_return_val_if_fail (link != NULL, 0);
+
+        return link->type;
+}
+
+/**
+ * dh_link_get_flags:
+ * @link: a #DhLink.
+ *
+ * Returns: the #DhLinkFlags of @link.
+ */
+DhLinkFlags
+dh_link_get_flags (DhLink *link)
+{
+        g_return_val_if_fail (link != NULL, DH_LINK_FLAGS_NONE);
+
+        return link->flags;
+}
+
+/**
+ * dh_link_set_flags:
+ * @link: a #DhLink.
+ * @flags: the new flags of the link.
+ *
+ * Sets the flags of the link.
+ */
+void
+dh_link_set_flags (DhLink      *link,
+                   DhLinkFlags  flags)
+{
+        g_return_if_fail (link != NULL);
+
+        link->flags = flags;
+}
+
 /**
  * dh_link_get_name:
  * @link: a #DhLink.
@@ -249,43 +224,11 @@ dh_link_unref (DhLink *link)
 const gchar *
 dh_link_get_name (DhLink *link)
 {
+        g_return_val_if_fail (link != NULL, NULL);
+
         return link->name;
 }
 
-/**
- * dh_link_get_book_name:
- * @link: a #DhLink.
- *
- * Returns: the name of the book that the @link is contained in.
- */
-const gchar *
-dh_link_get_book_name (DhLink *link)
-{
-        if (link->book != NULL) {
-                return link->book->name;
-        }
-
-        return "";
-}
-
-/**
- * dh_link_get_page_name:
- * @link: a #DhLink.
- *
- * Returns: the name of the page that the @link is contained in.
- * Deprecated: 3.26: This function is used nowhere and is actually broken, it
- * returns the book name.
- */
-const gchar *
-dh_link_get_page_name (DhLink *link)
-{
-        if (link->page != NULL) {
-                return link->page->name;
-        }
-
-        return "";
-}
-
 /**
  * dh_link_get_file_name:
  * @link: a #DhLink.
@@ -295,33 +238,13 @@ dh_link_get_page_name (DhLink *link)
 const gchar *
 dh_link_get_file_name (DhLink *link)
 {
+        g_return_val_if_fail (link != NULL, NULL);
+
         /* Return filename if the link is itself a page or if the link is within
-         * a page.
+         * a page (i.e. every link type except a book).
          */
-        if (link->page != NULL ||
-            link->type == DH_LINK_TYPE_PAGE) {
+        if (link->type != DH_LINK_TYPE_BOOK)
                 return link->relative_url;
-        }
-
-        return "";
-}
-
-/**
- * dh_link_get_book_id:
- * @link: a #DhLink.
- *
- * Returns: the book ID.
- */
-const gchar *
-dh_link_get_book_id (DhLink *link)
-{
-        if (link->type == DH_LINK_TYPE_BOOK) {
-                return link->book_id;
-        }
-
-        if (link->book) {
-                return link->book->book_id;
-        }
 
         return "";
 }
@@ -340,6 +263,8 @@ dh_link_get_uri (DhLink *link)
         gchar *uri;
         gchar *anchor;
 
+        g_return_val_if_fail (link != NULL, NULL);
+
         if (link->type == DH_LINK_TYPE_BOOK)
                 base_path = link->base_path;
         else
@@ -362,108 +287,175 @@ dh_link_get_uri (DhLink *link)
                 g_free (uri);
                 uri = uri_with_anchor;
         }
-        g_free (filename);
 
+        g_free (filename);
         return uri;
 }
 
 /**
- * dh_link_get_link_type:
+ * dh_link_get_book_name:
  * @link: a #DhLink.
  *
- * Returns: the #DhLinkType of @link.
- */
-DhLinkType
-dh_link_get_link_type (DhLink *link)
-{
-        return link->type;
-}
-
-/**
- * dh_link_get_flags:
- * @link: a #DhLink.
- *
- * Returns: the #DhLinkFlags of @link.
- */
-DhLinkFlags
-dh_link_get_flags (DhLink *link)
-{
-        return link->flags;
-}
-
-/**
- * dh_link_set_flags:
- * @link: a #DhLink.
- * @flags: the new flags of the link.
- *
- * Sets the flags of the link.
- */
-void
-dh_link_set_flags (DhLink      *link,
-                   DhLinkFlags  flags)
-{
-        link->flags = flags;
-}
-
-/**
- * dh_link_get_type_as_string:
- * @link: a #DhLink.
- *
- * Returns: a string representation of the #DhLinkType of @link, translated in
- * the current language.
+ * Returns: the name of the book that the @link is contained in.
  */
 const gchar *
-dh_link_get_type_as_string (DhLink *link)
+dh_link_get_book_name (DhLink *link)
 {
-        switch (link->type) {
+        g_return_val_if_fail (link != NULL, NULL);
+
+        if (link->book != NULL)
+                return link->book->name;
+
+        return "";
+}
+
+/**
+ * dh_link_get_book_id:
+ * @link: a #DhLink.
+ *
+ * Returns: the book ID.
+ */
+const gchar *
+dh_link_get_book_id (DhLink *link)
+{
+        g_return_val_if_fail (link != NULL, NULL);
+
+        if (link->type == DH_LINK_TYPE_BOOK)
+                return link->book_id;
+
+        if (link->book != NULL)
+                return link->book->book_id;
+
+        return "";
+}
+
+/**
+ * dh_link_compare:
+ * @a: (type DhLink): a #DhLink.
+ * @b: (type DhLink): a #DhLink.
+ *
+ * Compares the links @a and @b. This function is used to determine in which
+ * order the links should be displayed.
+ *
+ * Returns: an integer less than zero if @a should appear before @b; zero if
+ * there are no preferences; an integer greater than zero if @b should appear
+ * before @a.
+ */
+gint
+dh_link_compare (gconstpointer a,
+                 gconstpointer b)
+{
+        DhLink *la = (DhLink *) a;
+        DhLink *lb = (DhLink *) b;
+        gint flags_diff;
+        gint diff;
+
+        g_return_val_if_fail (a != NULL, 0);
+        g_return_val_if_fail (b != NULL, 0);
+
+        /* Sort deprecated hits last. */
+        flags_diff = ((la->flags & DH_LINK_FLAGS_DEPRECATED) -
+                      (lb->flags & DH_LINK_FLAGS_DEPRECATED));
+        if (flags_diff != 0)
+                return flags_diff;
+
+        /* Collation-based sorting */
+        if (G_UNLIKELY (la->name_collation_key == NULL))
+                la->name_collation_key = g_utf8_collate_key (la->name, -1);
+        if (G_UNLIKELY (lb->name_collation_key == NULL))
+                lb->name_collation_key = g_utf8_collate_key (lb->name, -1);
+
+        diff = strcmp (la->name_collation_key,
+                       lb->name_collation_key);
+
+        /* For the same names, sort page links before other links. The page is
+         * more important than a symbol (typically contained in that page).
+         */
+        if (diff == 0) {
+                if (la->type == lb->type)
+                        return 0;
+
+                if (la->type == DH_LINK_TYPE_PAGE)
+                        return -1;
+
+                if (lb->type == DH_LINK_TYPE_PAGE)
+                        return 1;
+
+                return 0;
+        }
+
+        return diff;
+}
+
+/**
+ * dh_link_type_to_string:
+ * @link_type: a #DhLinkType.
+ *
+ * Returns: a string representation of the #DhLinkType, translated in the
+ * current language.
+ */
+const gchar *
+dh_link_type_to_string (DhLinkType link_type)
+{
+        switch (link_type) {
         case DH_LINK_TYPE_BOOK:
                 /* i18n: a documentation book */
                 return _("Book");
+
         case DH_LINK_TYPE_PAGE:
                 /* i18n: a "page" in a documentation book */
                 return _("Page");
+
         case DH_LINK_TYPE_KEYWORD:
                 /* i18n: a search hit in the documentation, could be a
                  * function, macro, struct, etc */
                 return _("Keyword");
+
         case DH_LINK_TYPE_FUNCTION:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Function");
+
         case DH_LINK_TYPE_STRUCT:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Struct");
+
         case DH_LINK_TYPE_MACRO:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Macro");
+
         case DH_LINK_TYPE_ENUM:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
-               return _("Enum");
+                return _("Enum");
+
         case DH_LINK_TYPE_TYPEDEF:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Type");
+
         case DH_LINK_TYPE_PROPERTY:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Property");
+
         case DH_LINK_TYPE_SIGNAL:
                 /* i18n: in the programming language context, if you don't
                  * have an ESTABLISHED term for it, leave it
                  * untranslated. */
                 return _("Signal");
+
         default:
                 break;
         }
 
-        return "";
+        g_return_val_if_reached ("");
 }
