@@ -17,8 +17,8 @@
  * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __DH_PREFERENCES_H__
-#define __DH_PREFERENCES_H__
+#ifndef DH_PREFERENCES_H
+#define DH_PREFERENCES_H
 
 #include <gtk/gtk.h>
 
@@ -51,4 +51,4 @@ void dh_preferences_show_dialog (GtkWindow *parent);
 
 G_END_DECLS
 
-#endif /* __DH_PREFERENCES_H__ */
+#endif /* DH_PREFERENCES_H */
